
from PySteppables import *
import CompuCell
import sys
import random
from math import sqrt,pi,cos,sin
import numpy as np
from PySteppablesExamples import MitosisSteppableBase


class ConstraintInitializerSteppable(SteppableBasePy):
	def __init__(self,simulator,frequency,cell_size,lambdacell,targetCell):
		SteppableBasePy.__init__(self,simulator,frequency)
		self.cell_size=cell_size
		self.lambdacell=lambdacell
		self.targetCell=targetCell

	def start(self):
		###defining center of the lattice (xpos,ypos) to create the mother cell
		xpos=(self.dim.x)/2.
		ypos=(self.dim.y)/2.

		stem_cell=self.newCell(self.STEM)
		self.cellField[xpos-self.cell_size:xpos+self.cell_size-1,ypos-self.cell_size:ypos+self.cell_size-1,0]=stem_cell
		stem_cell.targetVolume=self.targetCell
		stem_cell.lambdaVolume=self.lambdacell
		##three attributes
		stem_cell.dict['list'] = [stem_cell.id]
		#time initial
		stem_cell.dict['list_time']=0
		#number of divisions from original
		stem_cell.dict['num_div']=0
		stem_cell.dict['color']=0.0

class GrowthSteppable(SteppableBasePy):
	def __init__(self,_simulator,_frequency=1):
		SteppableBasePy.__init__(self,_simulator,_frequency)
	def start(self):
		print "this argument is related to the growth command"
		print "user inputs whether they want uniform or contact inhibited growth"
		print "input either 0 for uniform or 1 for contact inhibited growth"
		self.growth_input=int(input("input 0 or 1:"))
	def step(self,mcs):
		if(self.growth_input==0):
			for cell in self.cellListByType(self.STEM):
				cell.targetVolume+=0.5
		elif(self.growth_input==1):
			for cell in self.cellListByType(self.STEM):
				for neighbor in self.getCellNeighbors(cell):
					if(not neighbor.neighborAddress):
						cell.targetVolume+=1.0


class DeathSteppable(SteppableBasePy):
	def __init__(self,_simulator,_frequency=1):
		SteppableBasePy.__init__(self,_simulator,_frequency)
	def start(self):
		print "this argument is related to the death command"
		print "user inputs whether they want cells to die after an interval or not"
		print "input either 0 for no death or 1 for death"
		self.death_input=int(input("input 0 or 1:"))
	def step(self,mcs):
		xcom=self.dim.x/2.
		ycom=self.dim.y/2.
		cells_to_kill=[]
		if(self.death_input==1 and mcs>=1500):
			for cell in self.cellListByType(self.STEM):
				dist = sqrt((xcom - cell.xCOM) ** 2 + (ycom - cell.yCOM) ** 2)
				if(dist<20):
					cells_to_kill.append(cell.id)

			for cellid in cells_to_kill:
				cell = self.attemptFetchingCellById(cellid)
				cell.targetVolume=0
class MitosisSteppable(MitosisSteppableBase):
	# In this class we check if cells reached twice its original volume. When it happens, we divide them
	# since each cell has a genetic network inside it, the values from the parent cell must be copied to
	# the daughter cell when they divide. The SBML model is copied
	def __init__(self, simulator, frequency, cell_size,targetcell,lambdacell):
		MitosisSteppableBase.__init__(self, simulator, frequency)
		self.cell_size=cell_size
		self.targetcell=targetcell
		self.mitosisVolume = 2.0 * targetcell
		self.lambdacell = lambdacell
		self.mcs=0

	def start(self):
		pass
	def step(self,mcs):
		self.mcs=mcs
		cells_to_divide = []
		for cell in self.cellList:
			if cell.volume > self.mitosisVolume:
				cells_to_divide.append(cell)

		for cell in cells_to_divide:
			# to change mitosis mode leave one of the below lines uncommented
			self.divideCellRandomOrientation(cell)
			# self.divideCellOrientationVectorBased(cell,1,0,0)                 # this is a valid option
			# self.divideCellAlongMajorAxis(cell)                               # this is a valid option
			# self.divideCellAlongMinorAxis(cell)                               # this is a valid option
	def updateAttributes(self):
		self.parentCell.targetVolume = self.targetcell  # reducing parent target volume
		self.cloneParent2Child()
		self.childCell.dict['list'].append(self.childCell.id)
		self.childCell.dict['list_time']=self.mcs#might be tricky
		self.childCell.dict['num_div']=self.parentCell.dict['num_div']+1
		self.childCell.dict['color']=self.parentCell.dict['color']
class histocells(SteppableBasePy):
	def __init__(self,simulator,frequency):
		SteppableBasePy.__init__(self,simulator,frequency)
		self.cellidField2 = self.createScalarFieldCellLevelPy("cellid")
	def start(self):
		###########################################################
		###################call plot properties###################
		self.pW = self.addNewPlotWindow(_title='Lineage Display', _xAxisTitle='cell counter',
										 _yAxisTitle='# of cells',
											 _xScaleType='linear', _yScaleType='linear')
		# length measurements
		self.pW.addPlot('Lineage', _style='Dots', _color='red', _size=5)
		##########################################################
		########################################################
		###############at the beginning of each simulation,each user
		##############will input a list of times at which they want to color
		########################################################
		print "User input when you want to display lineage output"
		self.time_to_display=int(input("enter time period of first display here:"))
		self.cell_list=[]
		self.temp_l=[]
	def step(self,mcs):
		for cell in self.cellListByType(self.STEM):
			self.cellidField2[cell]=cell.dict['color']
		#self.tm = self.time_list[self.tm_c]
		if(mcs==self.time_to_display-1):
			#######################################
			########make sure data is erased
			#####################################################
			self.pW.eraseAllData()
			for cell in self.cellListByType(self.STEM):
				#self.cellidField2[cell] = 0
				cell.dict['color']=0.0
				self.cellidField2[cell] = cell.dict['color']
			#######################################
			print "User enters what cell lineage criteria by picking number from 1 to 4"
			print "Criteria 1: Pick cells which fall between certain radii ranges"
			print "Criteria 2: Pick cells which fall beyond a certain radius"
			print "Criteria 3: Pick cells which divided between a time interval"
			print "Criteria 4: Pick cells which divided after a time interval"
			num=int(raw_input("Enter a number between 1 to 4 here:"))
			if(num==1):
				print "User inputs min max radius"
				min_radius=int(raw_input("enter minimum radius here:"))
				max_radius=int(raw_input("enter maximum radius here:"))
				self.cell_list=self.cells_at_radius(min_radius,max_radius)
			elif(num==2):
				print "User inputs min max radius"
				min_radius = int(raw_input("enter minimum radius here:"))
				self.cell_list=self.cells_beyond_radius(min_radius)
			elif(num==3):
				print "User inputs division time interval"
				time_interval= int(raw_input("enter time interval here:"))
				self.cell_list=self.cells_within_time(time_interval)
			elif(num==4):
				print "User inputs time interval beyond"
				time_interval = int(raw_input("enter time interval here:"))
				self.cell_list=self.cells_beyond_time(time_interval)
			self.temp_l=self.unique_parents(self.cell_list)

			##########################################################################
			##############Now that we have unique parents, we color cells belonging
			##############to a particular lineage
			#########################################################################
			#print self.temp_l
			self.temp_l.sort()##we sort the list so lower id cells are colored in shades of blue, high red
			ll = len(self.temp_l)##finds the length of the list
			counter = 1
			num_cell_list=[]
			for id in self.temp_l:
				cell_main = self.attemptFetchingCellById(id)
				counter_c = 0
				group = []
				# append self cell
				group.append(id)
				counter_c += 1
				for cell in self.cellListByType(self.STEM):
					if (id != cell.id and id in cell.dict['list']):
						if (cell.dict['num_div'] >= cell_main.dict['num_div']):
							group.append(cell.id)
							counter_c += 1
				#num_cells.append(counter_c)
				for cell2id in group:
					cell2 = self.attemptFetchingCellById(cell2id)
					cell2.dict['color']=((counter / float(ll))+0.1)/1.1
					self.cellidField2[cell2]=cell2.dict['color']
					#print 'value',self.cellidField2[cell2]
				counter += 1
				num_cell_list.append((id,counter_c))
				#self.pW.addDataPoint('Lineage',id,counter_c)
			#print 'before',num_cell_list
			num_cell_list.sort(key=lambda tup: tup[1])
			#num_cell_list.reverse()
			#print 'after',num_cell_list
			dis_c=1
			for (id,len_grp) in num_cell_list:
				self.pW.addDataPoint('Lineage',dis_c,len_grp)
				dis_c+=1
		if(mcs==self.time_to_display+1):
#			time.sleep(20)
			print "user decides how many more steps the simulation is run"
			print "display will stay on till next time input"
			tm_to_incr=int(input("enter how many more steps to run :"))
			self.time_to_display+=tm_to_incr
	#####################################################################
	######################Selecting Lineage Display Criteria#############
	#####################################################################
	#####################################################################
	################This Function Picks Cells Roughly At a Radius
	################From Center
	#####################################################################
	def cells_at_radius(self,min_radius,max_radius):
		xcom = self.dim.x / 2.  ##defining the x center of lattice to calculate dist from
		ycom = self.dim.y / 2.  ##defining the y center of lattice to calculate dist from
		############For user defined minimum and maximum radii,
		############we pick 80 points to define which radii to search at
		r_arr = np.linspace(min_radius, max_radius, 80)
		############To parse over the angles, we define an array containing
		###########values from 0 to 2pi. 500 points are picked
		theta_arr = np.linspace(0, 360, 500)
		cell_list = [] ###########cell_list will contain all the cells picked at the radii
		for theta in theta_arr:##loop to parse over angles
			for radius in r_arr:##for each angle, loop over radii
				xdist = xcom + radius * cos(theta * pi / 180.)##x coordinates from center are calculated
				ydist = ycom + radius * sin(theta * pi / 180.)##y coordinates from center are calculated
				xcoor = int(xdist)##converted to integer values to find them on the lattice
				ycoor = int(ydist)##converted to integer values to find them on the lattice
				pt = CompuCell.Point3D()##defines a lattice vector in CC3D
				pt.x = xcoor##specifying the x coordinates of the vector
				pt.y = ycoor##specifying the y coordinates of the vector
				pt.z = 0##in 2d setting the z coordinates to 0
				cell = self.cellField.get(pt)##this is to access the cell occupying the lattice point
				if (cell and cell.type != self.MEDIUM):##this is to make sure we don't choose a medium cell
					if (cell.id not in cell_list and cell.id != 1):##also avoid picking up cell.id=1
																	## since it created all daughter cells
						cell_list.append(cell.id)##Append all the cells chosen to the list
		return cell_list##This function will return the list of our chosen cells

	#####################################################################
	################This Function will find all cells whose center of mass
	################are beyond a particular radius
	#####################################################################
	def cells_beyond_radius(self,min_radius):
		xcom=self.dim.x/2.##defining the x center of lattice to calculate dist from
		ycom=self.dim.y/2.##defining the y center of lattice to calculate dist from
		cell_list=[]###########cell_list will contain all the cells picked beyond the radii
		for cell in self.cellListByType(self.STEM):##parsing through all cells of type STEM
			dist=sqrt((xcom-cell.xCOM)**2+(ycom-cell.yCOM)**2)##calculating dist from lattice center
			if(dist>=min_radius):##only pick cells which exceed the min radius
				if (cell.id not in cell_list and cell.id != 1):##avoid cellid=1 since it created all daughter cells
					cell_list.append(cell.id)##Append all the cells chosen to the list
		return cell_list##This function will return the list of our chosen cells

	#####################################################################
	################This Function will find all cells which divided
	################within a particular user defined time interval
	#####################################################################
	def cells_within_time(self,time_interval):
		cell_list=[]###########cell_list will contain all the cells which divided within the time interval
		for cell in self.cellListByType(self.STEM):##parsing through all cells of type STEM
			if(time_interval-200<cell.dict['list_time']<time_interval+200):##defining cells which divided within 200 mcs
																			##of the time interval
				if (cell.id not in cell_list and cell.id != 1):####avoid cellid=1 since it created all daughter cells
					cell_list.append(cell.id)##Append all the cells chosen to the list
		return cell_list##This function will return the list of our chosen cells
	####################################################################

	#####################################################################
	################This Function will find all cells which divided
	################within a particular user defined time interval
	#####################################################################
	def cells_beyond_time(self,time_interval):
		cell_list = []###########cell_list will contain all the cells which divided after a certain time
		for cell in self.cellListByType(self.STEM):##parsing through all cells of type STEM
			if (cell.dict['list_time'] > time_interval ):##defining cells which divided beyond a certain time
				if (cell.id not in cell_list and cell.id != 1):##avoid cellid=1 since it created all daughter cells
					cell_list.append(cell.id)##Append all the cells chosen to the list
		return cell_list##This function will return the list of our chosen cells
	########################################################################

	#########################################################################
	###############Making sure all the cells chosen are not descendents of
	###############each other, i.e looking for unique lineages###############
	########################################################################

	########################################################################
	###################This function will extract unique ids from the cell list
	########################################################################
	def unique_parents(self,cell_list):
		ele_to_delete=[]
		for cellid in cell_list:##parsing through cells chosen according to criteria
			for cellid2 in cell_list:##for each cell, parsing through the list again
				cell = self.attemptFetchingCellById(cellid2)##this will return the cell object at a particular id
				if (cellid != cellid2 and cellid in cell.dict['list']):##this checks any parent lineage id is present in 'list'
					if ((cellid2 > cellid) and (cellid2 not in ele_to_delete)):##only cells with higher id numbers are eliminated
																				##as they created later and make sure we don't
																				##recollect them
						ele_to_delete.append(cellid2)##we append all the cells to be deleted to the list called ele_to_delete

		temp_l = cell_list[:]  #copy list by reference to avoid deleting from main list
		for cellif in ele_to_delete:##parse through list
			temp_l.remove(cellif)##delete specific id by using 'remove'

		return temp_l

